﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Sound : MonoBehaviour
{
    public float sampleRate = 44100;
    public float waveLengthInSeconds = 2.0f;

    private int timeIndex = 0;
    private AudioSource audioSource;
    private AudioSource seAudioSource;
    private LevelTimer timer;

    private AudioClip soundWaitingForBomb;
    private AudioClip soundBomb;
    private AudioClip soundAlert;
    private AudioClip soundEnd;
    
    // Start is called before the first frame update
    void Start()
    {
        seAudioSource = this.GetComponent<AudioSource>();
        audioSource = gameObject.AddComponent<AudioSource>();
        audioSource.playOnAwake = false;
        audioSource.spatialBlend = 0; //force 2D sound
        audioSource.Stop(); //avoids audiosource from starting to play automatically

        soundWaitingForBomb = (AudioClip)Resources.Load("Sound/waiting_for_bomb");
        soundBomb = (AudioClip)Resources.Load("Sound/bomb");
        soundAlert = (AudioClip)Resources.Load("Sound/alert");
        soundEnd = (AudioClip)Resources.Load("Sound/end");
        timer = GameObject.Find("TimerHolder").GetComponent<LevelTimer>();
    }

    // Update is called once per frame
    void Update()
    {
        if (timer.waitingForBomb()){
            switchSound(seAudioSource, soundWaitingForBomb);
        }
        if (timer.buildOK())
        {
            switchSound(seAudioSource,soundBomb);
        }
        if (timer.alertStart())
        {
            switchSound(audioSource,soundAlert);
        }
        if (timer.stabilityCheck())
        {
            audioSource.Stop();
        }
        if (timer.levelComplete())
        {
            switchSound(audioSource,soundEnd);
        }
    }

    private void switchSound(AudioSource source, AudioClip clip){
        if(source.clip != clip){
            source.clip = clip;
            source.Play();
        }
    }
    /*
    private void OnAudioFilterRead(float[] data, int channels)
    {
        float frequency = timeToFrequency(timeIndex);
        Debug.Log("timer:" + timer.timer);
        float detuneRatio = 0.01f;

        for(int i = 0; i < data.Length; i+= channels)
        {
            data[i] = CreateSine(timeIndex, frequency * (1.0f + detuneRatio), sampleRate);

            if(channels == 2)
                data[i+1] = CreateSine(timeIndex, frequency * (1.0f - detuneRatio), sampleRate);

            timeIndex++;

            //if timeIndex gets too big, reset it to 0
            if(timeIndex >= (sampleRate * waveLengthInSeconds))
            {
                timeIndex = 0;
            }
        }
    }
    */
    private float CreateSine(int timeIndex, float frequency, float sampleRate)
    {
        return Mathf.Sin(2 * Mathf.PI * timeIndex * frequency / sampleRate);
    }

    private float timeToFrequency(int timeIndex)
    {
        return 440.0f;
    }
}
